# -*- coding: utf-8 -*-
"""
Created on Wed Feb 12 09:52:14 2020
@author: Gareth V. Walkom (walkga04 at googlemail.com)

Change these roots to your own.

"""
import datetime

# Define Calibration Data Root
DATA = 'F:/LMK/Calibration Data'

# Define Save Root
SAVE = 'E:/Measurements/' + str(datetime.date.today()) + '/'
SAVE_TEST = 'E:/Measurements/TEST/'

# Define Load Root
LOAD = 'E:/Measurements/2020-03-05/'
LOAD_HYP = 'F:/Program Files/Hyperspectral Camera/GUIHSK23052016/Data_Cube/20200305_081505/'
